import { Injectable } from "@angular/core";
import { DataParser } from "../util/Dataparser";
import { makeTestnetWallet } from "../util/TestnetUtils";

declare const xrpIO: any
declare const xrpl: any

const xrpNode = "wss://s.altnet.rippletest.net:51233"

@Injectable()
export class ShoutboxDataService {

    public history: any[] = []

    private userWallet1: any
    private userWallet2: any
    private shoutboxAddress = "rBGqUNVzrNJ6CNpTriBts8QCTNi5VqUMnZ"
    private rippleApi: any;
    private xrpio: any;

    private getTransactions = async () => {
        const resp = await this.rippleApi.request({
            command: "account_tx",
            account: this.shoutboxAddress,
            forward: false,
        })
        return resp.result.transactions.map((entry: any) => entry.tx)
    }

    private submit = async (shoutHash: string) => {
        const data = { 
            endpoint: 'submit',
            data: shoutHash
        }

        console.log("Submitting shout root hash", data)
        const submitTx = await this.xrpio.writeRaw({data: JSON.stringify(data)}, this.shoutboxAddress, this.userWallet1.secret)
        
        console.log("submit TX @", submitTx)
        
        return submitTx
    }

    public submitShout = async (shout: any) => {
        console.log("writing shout data", shout)
        const shoutHash = await this.xrpio.treeWrite(JSON.stringify(shout), this.userWallet1.address, this.userWallet2.secret)
        console.log("shout data @ ", shoutHash)
        return await this.submit(shoutHash)
    }

    private parseMemos = async (memos: any) => {
        const shouts = await Promise.all(memos
            .map((memo: any) => {
                if (!memo.Memo || !memo.Memo.MemoData)
                    return

                try {
                    return DataParser.parse('TxHash', hex_to_ascii(memo.Memo.MemoData))
                } catch (e) {
                    return
                }
            })
            .filter((hash: string) => hash != undefined)
            .map((root_hash: string) => this.xrpio.treeRead([root_hash]))
        )
        return shouts.map((jsonStr: string) => JSON.parse(jsonStr))
    }

    private loadHistory = async () => {
        const raw_txs = await this.getTransactions()
        return await this.parseMemos(raw_txs.flatMap((htx: any) => htx.Memos))
    }

    private subscribeTxs = async (callback: Function) => {
        this.rippleApi.on('transaction', (tx: any) => callback(tx))
        await this.rippleApi.connection.request({
            command: 'subscribe',
            accounts: [this.shoutboxAddress]
        })
    }

    private listen = async () => {
        await this.subscribeTxs(async (raw_tx: any) => {
            const shouts = await this.parseMemos(raw_tx.transaction.Memos)
            this.history.unshift(...shouts)
        })
    }

    initialize = async () => {
        this.userWallet1 = await makeTestnetWallet()
        this.userWallet2 = await makeTestnetWallet()

        this.rippleApi = new xrpl.Client(xrpNode)
        await this.rippleApi.connect()

        this.xrpio = new xrpIO(xrpNode);
        await this.xrpio.connect()

        this.history = await this.loadHistory()

        await this.listen();
    }
}

export function initShoutboxSvc(svc: ShoutboxDataService): () => Promise<any> {
    return svc.initialize;
}

function hex_to_ascii(input: any) {
    var hex = input.toString();
    var str = '';
    for (var n = 0; n < hex.length; n += 2) {
        str += String.fromCharCode(parseInt(hex.substr(n, 2), 16));
    }
    return str;
}